@@ -2,6 +2,7 @@
 import pprint
 
 from typing import Any
+from typing import Dict
 
 import numpy as np
 import pandas as pd
@@ -10,9 +11,8 @@ from lmfit import Minimizer
 from lmfit import conf_interval
 from lmfit import report_ci
 from lmfit import report_fit
+from lmfit.minimizer import MinimizerException
 from lmfit.minimizer import minimize
-from lmfit.parameter import Parameters
-from lmfit.printfuncs import alphanumeric_sort
 from tabulate import tabulate
 
 
@@ -20,9 +20,7 @@ CORREL_HEAD = "[[Correlations]] (unreported correlations are < %.3f)"
 pp = pprint.PrettyPrinter(indent=4)
 
 
-def fit_report_as_dict(
-    inpars: minimize, modelpars: dict = None, sort_pars: bool = False
-) -> dict:
+def fit_report_as_dict(inpars: minimize, modelpars: dict = None) -> dict:
     """Generate the best fit report as dictionary.
 
     The report contains the best-fit values for the parameters and their
@@ -32,31 +30,14 @@ def fit_report_as_dict(
         inpars (minimize): Input Parameters from a fit or the  Minimizer results
              returned from a fit.
         modelpars (dict, optional): Known Model Parameters. Defaults to None.
-        sort_pars (bool, optional): Whether to show parameter names sorted in
-             alphanumerical order. If False (default), then the parameters will be
-             listed in the order they were buffered to the Parameters dictionary. If
-             callable, then this (one argument) function is used to extract a
-             comparison key from each list element. Defaults to False.
-
-    Raises:
-        SystemExit: In case of `numdifftools` is not installed, it can raise SystemExit.
 
     Returns:
         dict: [description]
     """
-    if isinstance(inpars, Parameters):
-        result, params = None, inpars
-    if hasattr(inpars, "params"):
-        result = inpars
-        params = inpars.params
+    result = inpars
+    params = inpars.params
 
-    if sort_pars:
-        key = sort_pars if callable(sort_pars) else alphanumeric_sort
-        parnames = sorted(params, key=key)
-    else:
-        # dict.keys() returns a KeysView in py3, and they're indexed
-        # further down
-        parnames = list(params.keys())
+    parnames = list(params.keys())
 
     buffer: dict = {
         "configurations": {},
@@ -86,11 +67,9 @@ def fit_report_as_dict(
         if not result.errorbars:
             print("##  Warning: uncertainties could not be estimated:")
             if result.method not in ("leastsq", "least_squares"):
-                raise SystemExit(
-                    "    this fitting method does not natively calculate uncertainties"
-                    "    and numdifftools is not installed for lmfit to do this. Use"
-                    "    `pip install numdifftools` for lmfit to estimate uncertainties"
-                    "    with this fitting method."
+                print(
+                    f"The fitting method '{result.method}' does not natively calculate"
+                    " and uncertainties cannot be estimated due to be out of region!"
                 )
 
             parnames_varying = [par for par in result.params if result.params[par].vary]
@@ -143,58 +122,89 @@ def fit_report_as_dict(
             buffer["covariance_matrix"][name_1] = {}
             for j, name_2 in enumerate(parnames):
                 buffer["covariance_matrix"][name_1][name_2] = result.covar[i, j]
-    except AttributeError as exc:
+    except (AttributeError, IndexError) as exc:
         print(f"{exc}: Covariance Matrix could not be calculated.\n")
     return buffer
 
 
-def printing_regular_mode(
-    args: dict, result: Any, minimizer: Minimizer, correlation: pd.DataFrame
-) -> None:
-    """Printing the fitting results in the regular mode.
+class PrintingResults:
+    """Print the results of the fitting process."""
 
-    Args:
-        args (dict): The input file arguments as a dictionary with additional
-             information beyond the command line arguments.
-        result (Any): The lmfit `results` as a kind of result based class.
-        minimizer (Minimizer): The lmfit `Minimizer`-class as a general minimizer for
-             curve fitting and optimization.
-        correlation (pd.DataFrame): The correlation results of the global fit as pandas
-             DataFrame.
-    """
-    print("\nStatistic:\n")
-    print(
-        tabulate(
-            pd.DataFrame.from_dict(args["data_statistic"]),
-            headers="keys",
-            tablefmt="fancy_grid",
-            floatfmt=".2f",
+    def __init__(
+        self,
+        args: Dict[str, Any],
+        result: Any,
+        minimizer: Minimizer,
+    ) -> None:
+        """Initialize the PrintingResults class.
+
+        Args:
+            args (Dict[str,Any]): The input file arguments as a dictionary with
+                 additional information beyond the command line arguments.
+            result (Any): The lmfit `results` as a kind of result based class.
+            minimizer (Minimizer): The lmfit `Minimizer`-class as a general minimizer
+                 for curve fitting and optimization.
+        """
+        self.args = args
+        self.result = result
+        self.minimizer = minimizer
+        self.correlation = pd.DataFrame.from_dict(args["linear_correlation"])
+
+    def __call__(self) -> None:
+        """Print the results of the fitting process."""
+        if self.args["verbose"]:
+            self.printing_verbose_mode
+        else:
+            self.printing_regular_mode
+
+    @property
+    def printing_regular_mode(self) -> None:
+        """Print the fitting results in the regular mode."""
+        print("\nStatistic:\n")
+        print(
+            tabulate(
+                pd.DataFrame.from_dict(self.args["data_statistic"]),
+                headers="keys",
+                tablefmt="fancy_grid",
+                floatfmt=".2f",
+            )
+        )
+        print("\nFit Results and Insights:\n")
+        print(
+            report_fit(self.result, modelpars=self.result.params, **self.args["report"])
+        )
+        if self.args["conf_interval"]:
+            print("\nConfidence Interval:\n")
+            try:
+                report_ci(
+                    conf_interval(
+                        self.minimizer, self.result, **self.args["conf_interval"]
+                    )
+                )
+            except MinimizerException as exc:
+                print(f"Error: {exc} -> No confidence interval could be calculated!")
+                self.args["confidence_interval"] = None
+        print("\nOverall Linear-Correlation:\n")
+        print(
+            tabulate(
+                pd.DataFrame.from_dict(self.args["linear_correlation"]),
+                headers="keys",
+                tablefmt="fancy_grid",
+                floatfmt=".2f",
+            )
         )
-    )
-    print("\nFit Results and Insights:\n")
-    print(report_fit(result, modelpars=result.params, **args["report"]))
-    if args["conf_interval"]:
-        print("\nConfidence Interval:\n")
-        report_ci(conf_interval(minimizer, result, **args["conf_interval"]))
-    print("\nOverall Linear-Correlation:\n")
-    print(tabulate(correlation, headers="keys", tablefmt="fancy_grid", floatfmt=".2f"))
 
-
-def printing_verbose_mode(args: dict) -> None:
-    """Printing all results in verbose mode.
-
-    Args:
-        args (dict): The input file arguments as a dictionary with additional
-             information beyond the command line arguments.
-    """
-    print("\nStatistic:\n")
-    pp.pprint(args["data_statistic"])
-    print("Input Parameter:\n")
-    pp.pprint(args)
-    print("\nFit Results and Insights:\n")
-    pp.pprint(args["fit_insights"])
-    if args["conf_interval"]:
-        print("\nConfidence Interval:\n")
-        pp.pprint(args["confidence_interval"])
-    print("\nOverall Linear-Correlation:\n")
-    pp.pprint(args["linear_correlation"])
+    @property
+    def printing_verbose_mode(self) -> None:
+        """Print all results in verbose mode."""
+        print("\nStatistic:\n")
+        pp.pprint(self.args["data_statistic"])
+        print("Input Parameter:\n")
+        pp.pprint(self.args)
+        print("\nFit Results and Insights:\n")
+        pp.pprint(self.args["fit_insights"])
+        if self.args["conf_interval"]:
+            print("\nConfidence Interval:\n")
+            pp.pprint(self.args["confidence_interval"])
+        print("\nOverall Linear-Correlation:\n")
+        pp.pprint(self.args["linear_correlation"])
